@@ -1,5 +1,6 @@
 """Testing serialization of DeltaGraph and its componenents."""
 
+import json
 import unittest
 from unittest.mock import Mock
 
@@ -9,7 +10,11 @@ import dill
 import deltalanguage.data_types.dotdf_capnp \
     as dotdf_capnp  # pylint: disable=E0401, disable=E0611
 from deltalanguage._utils import NamespacedName
-from deltalanguage.data_types import DOptional, DInt, NoMessage, as_delta_type
+from deltalanguage.data_types import (BaseDeltaType,
+                                      DOptional,
+                                      DInt,
+                                      NoMessage,
+                                      as_delta_type)
 from deltalanguage.runtime import (DeltaRuntimeExit,
                                    deserialize_graph,
                                    serialize_graph)
@@ -18,6 +23,7 @@ from deltalanguage.wiring import (DeltaBlock,
                                   InPort,
                                   Interactive,
                                   OutPort,
+                                  PythonBody,
                                   PySplitterBody,
                                   template_node_factory)
 
@@ -26,7 +32,6 @@ from deltalanguage.lib.quantum_simulators import (ProjectqQuantumSimulator,
                                                   QiskitQuantumSimulator)
 
 
-
 class PortSerialisationTest(unittest.TestCase):
 
     def test_in_port_capnp(self):
@@ -287,17 +292,20 @@ class PythonNodeSerialisationTest(unittest.TestCase):
             self.func(40, 2)
 
         data, _ = serialize_graph(graph)
-        g_capnp = deserialize_graph(data)
-        g_capnp_list = str(g_capnp).split('\n')
-
-        with open('test/data/graph.df', 'r') as file:
-            g_str_list = file.read().split('\n')
-
         self.assertEqual(type(data), bytes)
-        self.assertEqual(len(g_capnp_list), len(g_str_list))
-        for l1, l2 in zip(g_capnp_list, g_str_list):
-            if not 'dillImpl' in l1:
-                self.assertEqual(l1, l2)
+        g_capnp = deserialize_graph(data).to_dict()
+        for body in g_capnp['bodies']:
+            if 'python' in body:
+                self.assertTrue(isinstance(dill.loads(
+                    body['python']['dillImpl']), PythonBody))
+                del body['python']['dillImpl']
+        for node in g_capnp['nodes']:
+            for port in node['inPorts'] + node['outPorts']:
+                self.assertTrue(isinstance(
+                    dill.loads(port['type']), BaseDeltaType))
+                del port['type']
+        with open('test/data/graph_capnp.json', 'r') as file:
+            self.assertEqual(g_capnp, json.load(file))
 
 
 if __name__ == "__main__":
